MODULE SysLog;

(*
MODULE:         System Log (Desktop Accessory)

AUTHOR:         John Michael Bennett
                12925 - 118th Avenue S.E.
                Rainier, Washington 98576
                (206)446-2427

LANGUAGE:       TDI Modula-2/ST, Release 3.01a
                0272-742796(UK), (214)340-4942(USA)

SYSTEM:         Atari 1040ST with MIND MINE internal clock, SH204, 
                SC1224 & SM124.

PURPOSE:        To keep track of how many hours are spent using the Hard
                Disk and to park the Hard Disk when logging off.  Time is
                shown in accumulated days, hours, and minutes.

EXTERNAL MODULES CALLED FROM STLIB:
  Bits, DateLib, MEMORY, STError, STIO, Timer.

STANDARD EXTERNAL MODULES CALLED:
  AESApplications, AESEvents, AESForms, AESGraphics, AESMenus, Conversions,
  GEMAESbase, GEMDOS, Keyboard, Strings, XBIOS

REVISION HISTORY:
   1.00  19-May-1987, John Michael Bennett.
         Original Release.
   1.01  07-Jul-1987, JMB.
         Modified Negative Time section of LOGOFF process.  If the
         time we log off is less than the time we logged on, the
         following steps will take place:

                1. Set session connect time to 0.
                2. Display a negative time not logged message.
                3. Set the logged on time = logged off time so
                   that next time we logon we will not have the
                   same negative time error for that session.

   1.02  27-Jul-1987, JMB.
         Changed alert box to display free memory with warning about
         parking heads.  This can be handy when wanting to know how
         much memory is free.  Sometimes the system just seems not to
         release memory after a program aborts.  Maybe that is not the
         reason but sometimes memory disappears for some unknown reason.
   1.10  09-Sep-1987, JMB.
         Added display of connect time to alert box. Added LOGON procedure
         to module instead of having it be a separate module executed from
         the auto folder during power up.  This makes it a simpler package
         to maintain.  Changed the user record to keep track of seconds.
   1.11  26-Sep-1987, JMB.
         If the current year (VIA GEMDOS.GetDate) is not the same as the
         year we last logged off (read from logfile) then we will allow
         the user to enter the current date.
         Changed "Press any key" routine on LOGON to continue if no key
         is pressed in 20 seconds.
   1.12  02-Oct-1987, JMB.
         Cleaned up code by creating more procedures.  Created timed
         input for starup screen.  Also added option keys to startup
         screen.  Added Help & Stats screens.  Expanded logfile to 
         contain total sessions, date of purchase.
   1.13  24-Oct-1987, JMB.
         Fixed resolution bug with color monitor.  Now restores old
         resolution and palette after logon is complete.
*)
(*$PAGE "MODULE LOGOFF"*)
(*$S-,$T-,$Q+*)

FROM Bits
     IMPORT ReSetL, TestL;

FROM STIO
     IMPORT Write, WriteString, WriteCard, WriteLn, WriteHex,
            WriteLongCard, WriteLongReal, ReadLine, ClearScreen,
            WriteLine, WriteCardUsing, GotoXY, STDIN, GetStatus,
            ReverseOn, ReverseOff, Read, ReadCard, ReadLongCard;

FROM Timer
     IMPORT Pause, InitTimer, CheckTimer;

FROM XBIOS
     IMPORT Palette, SetPalette, GetResolution, SetColour,
            ScreenPhysicalBase, ScreenLogicalBase, SetScreenBase;

IMPORT GEMDOS;

FROM MEMORY
     IMPORT SYSINFO;

FROM SYSTEM
     IMPORT ADDRESS, ADR, SETREG;

FROM DateLib
     IMPORT UnPackDate, PackDate, UnPackTime, PackTime,
            DateLine, TimeLine, Jdate, Date, Time, Getdate,
            GetIKBDdate, DateOk;

FROM STError
     IMPORT WriteError;

FROM Strings
     IMPORT Assign, Compare, CompareResults, Concat, Insert, Length,
            String;

FROM StrLib
     IMPORT Upshift;

FROM AESForms
     IMPORT FormAlert;
                     
FROM AESMenus
     IMPORT MenuRegister;

IMPORT Keyboard;

FROM AESEvents
     IMPORT EventMessage;

FROM GEMAESbase
     IMPORT AccessoryOpen, MouseOff, MouseOn;

FROM AESGraphics
     IMPORT GrafMouse;

FROM Conversions
     IMPORT ConvertToString;
     
FROM AESApplications
     IMPORT ApplInitialise, ApplExit;

CONST
  StackSize     = 8192;
  LogFile       = "C:\SYSLOG.DAT                 ";
  
TYPE
  ConnectType   = RECORD
                    Days    : CARDINAL;
                    Hours   : CARDINAL;
                    Minutes : CARDINAL;
                    Seconds : CARDINAL;
                  END;

  UserType      = RECORD
                    ConnectTime : ConnectType;
                    LoggedOn    : RECORD
                                    Date : CARDINAL;
                                    Time : CARDINAL;
                                  END;
                    LoggedOff   : RECORD
                                    Date : CARDINAL;
                                    Time : CARDINAL;
                                  END;
                    Purchased   : CARDINAL;                     (* 1.12 *)
                    Sessions    : LONGCARD;                     (* 1.12 *)
                  END;

VAR
  NegativeTime  : BOOLEAN;
  Stack         : ARRAY [0 .. StackSize] OF CARDINAL;
  user          : UserType;
  apid,
  menuid,
  resolution    : INTEGER;                                      (* 1.13 *)
  ScrnPalette   : Palette;                                      (* 1.13 *)
  LogHandle     : INTEGER;
  CurrentDate,
  CurrentTime   : CARDINAL;
  DateStr,
  str,
  accname       : String;
  fpos,
  len           : LONGCARD;
  ch, scan      : CHAR;


PROCEDURE ConfigScreen;

VAR
  x : CARDINAL;

BEGIN
  resolution := GetResolution();                                (* 1.13 *)

  FOR x := 0 TO 15 DO      (* Save screen palette *)            (* 1.13 *)
    ScrnPalette[ x ] := SetColour( x, CARDINAL( -1 ) );         (* 1.13 *)
  END;                                                          (* 1.13 *)
  
  IF resolution = 0 THEN
    SetScreenBase( NIL, NIL, 1 );
  END;

  IF resolution < 2 THEN
    x := SetColour( 0, 0 );     (* black background *)
    x := SetColour( 3, 630H );  (* amber color text *)
  ELSE
    x := SetColour( 0, 0 );     (* black background *)
    x := SetColour( 1, 777H );  (* white text *)
  END;

  ClearScreen;
END ConfigScreen;


PROCEDURE RestoreScreen;                                        (* 1.13 *)
                                                                (* 1.13 *)
VAR                                                             (* 1.13 *)
  x : CARDINAL;                                                 (* 1.13 *)
                                                                (* 1.13 *)
BEGIN                                                           (* 1.13 *)
  SetScreenBase( NIL, NIL, resolution );                        (* 1.13 *)
  SetPalette( ScrnPalette );                                    (* 1.13 *)
END RestoreScreen;                                              (* 1.13 *)


PROCEDURE Connected( date1, time1, date2, time2 : CARDINAL ) : LONGCARD;
(*
------------------------------------------------------------------------
-- This procedure will return the elapsed time in seconds between the --
-- date1, time1, and date2, time2.  If the elasped time is negative   --
-- then zero will be returned.                                        --
--------------------------------------------------------Version-1.10----
*)

VAR
  TDays, THours, TMinutes, TSeconds                         : LONGINT;
  monthOn, dayOn, yearOn, hourOn, minuteOn, secondOn        : CARDINAL;
  monthOff, dayOff, yearOff, minuteOff, secondOff, hourOff  : CARDINAL;
  jd1, jd2                                                  : LONGCARD;

BEGIN
  jd1 := LONGCARD( date1 ) * 65536 + LONGCARD( time1 );
  jd2 := LONGCARD( date2 ) * 65536 + LONGCARD( time2 );

  NegativeTime := ( jd2 < jd1 );
  IF NegativeTime THEN
    RETURN LONGCARD( 0 );
  ELSE
    UnPackDate( date1, monthOn, dayOn, yearOn );
    UnPackTime( time1, hourOn, minuteOn, secondOn );
  
    UnPackDate( date2, monthOff, dayOff, yearOff );
    UnPackTime( time2, hourOff, minuteOff, secondOff );

    jd1 := Jdate( monthOn, dayOn, yearOn );
    jd2 := Jdate( monthOff, dayOff, yearOff );
    
    TDays := LONGINT( jd2 ) - LONGINT( jd1 );
    THours := LONGINT( hourOff ) - LONGINT( hourOn );
    TMinutes := LONGINT( minuteOff ) - LONGINT( minuteOn );
    TSeconds := LONGINT( secondOff ) - LONGINT( secondOn );

    IF TSeconds < 0 THEN
      TSeconds := TSeconds + 60;
      TMinutes := TMinutes - 1;
    END;

    IF TMinutes < 0 THEN
      TMinutes := TMinutes + 60;
      THours := THours - 1;
    END;

    IF THours < 0 THEN
      THours := THours + 24;
      TDays := TDays - 1;
    END;
    RETURN LONGCARD( TSeconds + TMinutes * 60 + THours * 3600 + TDays * 86400 );
  END;
END Connected;


PROCEDURE InitLogFile;                                          (* 1.12 *)

VAR
  Month,
  Day,
  Year,
  THours : CARDINAL;

BEGIN
  WriteLn; WriteLn;
  WriteString("Enter the estimated number of hours you have used the system: ");
  THours := user.ConnectTime.Days * 24 + user.ConnectTime.Hours;

  user.ConnectTime.Days := 0;
  user.ConnectTime.Hours := 0;
  user.ConnectTime.Minutes := 0;
  user.ConnectTime.Seconds := 0;
  GEMDOS.GetDate( user.LoggedOn.Date );
  GEMDOS.GetTime( user.LoggedOn.Time );
  user.LoggedOff.Date := user.LoggedOn.Date;
  user.LoggedOff.Time := user.LoggedOn.Time;

  ReadCard( THours );

  user.ConnectTime.Days := THours DIV 24;
  user.ConnectTime.Hours := THours MOD 24;

  WriteString('Enter the estimated number of times you have used the system: ');
  ReadLongCard( user.Sessions );
  REPEAT
    WriteLn;
    WriteLine('Enter the date you purchased your HARD DISK DRIVE:');
    WriteString('  Month: ');
    ReadCard( Month );
    WriteString('  Day:   ');
    ReadCard( Day );
    WriteString('  Year:  ');
    ReadCard( Year );
    WriteLn;
  UNTIL PackDate( user.Purchased, Month, Day, Year ) = DateOk;
END InitLogFile;                                                (* 1.12 *)


PROCEDURE ShowLogInfo;                                          (* 1.12 *)

VAR
  THours,
  TMinutes : LONGCARD;

BEGIN
  ClearScreen;
  SYSINFO;
  WriteLn;
  WITH user.ConnectTime DO
    WriteString('ACCUMULATED CONNECT TIME = ');
    IF Days > 0 THEN
      WriteCard( Days, 1 ); WriteString(' DAY');
      IF Days <> 1 THEN Write("S"); END; WriteString("; ");
    END;
    THours := LONGCARD( Days ) * 24 + LONGCARD( Hours );
    TMinutes := THours * 60 + LONGCARD( Minutes );
    WriteLongCard( THours, 1 ); WriteString(' HOURS; ');
    WriteLongCard( TMinutes, 1 ); WriteString(' MINUTES.');
    WriteLn;
    WriteLn;
  END; (* WITH *)

  WriteString("LOGGED ON:  ");
  DateStr := '@'; str := '';
  DateLine( user.LoggedOn.Date, DateStr );
  Upshift( DateStr );
  TimeLine( user.LoggedOn.Time, str );
  Upshift( str );
  WriteString( str ); WriteString(", ");
  WriteString( DateStr ); WriteLn;

  WriteString("LOGGED OFF: ");
  DateStr := '@'; str := '';
  DateLine( user.LoggedOff.Date, DateStr );
  Upshift( DateStr );
  TimeLine( user.LoggedOff.Time, str );
  Upshift( str );
  WriteString( str ); WriteString(", ");
  WriteString( DateStr ); WriteLn; WriteLn;
END ShowLogInfo;                                                (* 1.12 *)


PROCEDURE Statistics;                                           (* 1.12 *)
(*
Purchased Hard Disk.............................: MON, FEB 13, 1987
Total Days since purchase of the hard disk......: 9012
Total Sessions..................................: 403
Average sessions per day........................: 1.82
Average connect time per session................: 103 DAYS, 01:05:02
Average connect time per day....................: 01:05:02
Connect Days....................................: 35
Connect Hours...................................: 904
Connect Minutes.................................: 59324
Connect Seconds.................................: 3559440
*)

VAR
  aspd : LONGREAL;
  TDays, THours, TMinutes, TSeconds, ASeconds, j1, j2, TotalDays : LONGCARD;
  Days, Hours, Minutes, Seconds, Month, Day, Year : CARDINAL;


  PROCEDURE WriteTime;

  VAR
    ADays, AHours, AMinutes : CARDINAL;

  BEGIN
    ADays := CARDINAL( ASeconds DIV 86400 );
    ASeconds := ASeconds MOD 86400;
    AHours := CARDINAL( ASeconds DIV 3600 );
    ASeconds := ASeconds MOD 3600;
    AMinutes := CARDINAL( ASeconds DIV 60 );
    ASeconds := ASeconds MOD 60;

    IF ADays > 0 THEN
      WriteCard( ADays, 1 );
      WriteString('DAY');
      IF TDays <> 1 THEN
        Write('S');
      END;
      WriteString(', ');
    END;

    WriteCardUsing( AHours, 2, '0'); Write(':');
    WriteCardUsing( AMinutes, 2, '0'); Write(':');
    WriteCardUsing( CARDINAL( ASeconds ), 2, '0');
    WriteLn;
  END WriteTime;


BEGIN
  WITH user.ConnectTime DO
    TDays := LONGCARD( Days );
    THours := LONGCARD( Days ) * 24 + LONGCARD( Hours );
    TMinutes := THours * 60 + LONGCARD( Minutes );
    TSeconds := TMinutes * 60 + LONGCARD( Seconds );
  END;

  WriteLine('LOG FILE STATISTICS:');
  WriteLn;

  WriteString('Purchased Hard Disk.........................: ');
  DateStr := '@';
  DateLine( user.Purchased, DateStr ); Upshift( DateStr );
  WriteLine( DateStr );

  WriteString('Total days since purchase of Hard Disk......: ');
  UnPackDate( user.Purchased, Month, Day, Year );
  j1 := Jdate( Month, Day, Year );
  GEMDOS.GetDate( CurrentDate );
  UnPackDate( CurrentDate, Month, Day, Year );
  j2 := Jdate( Month, Day, Year );
  TotalDays := j2 - j1;
  WriteLongCard( TotalDays, 1 ); WriteLn;

  WriteString('Total Sessions..............................: ');
  WriteLongCard( user.Sessions, 1 ); WriteLn;

  IF user.Sessions > TotalDays THEN
    WriteString('Average sessions per day....................: ');
    aspd := FLOATD( user.Sessions ) / FLOATD( TotalDays );
  ELSE
    WriteString('Average days per session....................: ');
    aspd := FLOATD( TotalDays ) / FLOATD( user.Sessions );
  END;
  WriteLongReal( aspd, 4, 2 );
  WriteLn;

  WriteString('Average connect time per session............: ');
  ASeconds := TSeconds DIV user.Sessions;
  WriteTime;

  WriteString('Average connect time per day................: ');
  ASeconds := TSeconds DIV TotalDays;
  WriteTime;

  WriteString('Connect Days................................: ');
  WriteLongCard( TDays, 1 ); WriteLn;

  WriteString('Connect Hours...............................: ');
  WriteLongCard( THours, 1 ); WriteLn;

  WriteString('Connect Minutes.............................: ');
  WriteLongCard( TMinutes, 1 ); WriteLn;
END Statistics;                                                 (* 1.12 *)


PROCEDURE LOGON;

VAR
  UpdateLogFile : BOOLEAN;
  creating      : BOOLEAN;
  Lon, Loff     : LONGCARD;
  
BEGIN
  UpdateLogFile := TRUE;
  GEMDOS.GetDate( CurrentDate );
  GEMDOS.GetTime( CurrentTime );
  ConfigScreen;

  GEMDOS.Open( LogFile, 2, LogHandle );
  creating := FALSE;

  IF LogHandle = -33 THEN
    GEMDOS.Create( LogFile, 0, LogHandle );
    creating := TRUE;
  END;

  IF LogHandle > 0 THEN
    len := LONGCARD( SIZE( user ) );

    IF creating THEN
      InitLogFile;
      GEMDOS.Write( LogHandle, len, ADR( user ) );
      IF GEMDOS.Close( LogHandle ) THEN END;
      GEMDOS.Open( LogFile, 2, LogHandle );
    END;
    
    GEMDOS.Read( LogHandle, len, ADR( user ) );

    LOOP;
      ShowLogInfo;
      WriteLn;
      WriteLine('  [D]    : Set todays date & time.');
      WriteLine('  [I]    : Re-initialize log file.');
      WriteLine('  [HELP] : Display program information.');
      WriteLn;
      WriteLine('  Press any other key to continue...');
      WriteLn;

      InitTimer;                                                (* 1.11 *)
      WHILE ( CheckTimer() < 60000 ) AND NOT GEMDOS.ConIS() DO
        WriteCard( 60 - CARDINAL( CheckTimer() ) DIV 1000, 2);
        WriteString(' seconds until GEM Desktop.'); Write( 15c );
      END;

      WriteLn;
      WriteLn;
      
      IF GEMDOS.ConIS() THEN                                    (* 1.11 *)
        GEMDOS.RawScanIn( ch, scan );                           (* 1.11 *)
        ch := CAP( ch );
        IF ch = 'D' THEN
          Date('');                                             (* 1.11 *)
          Time('');                                             (* 1.11 *)
          GEMDOS.GetDate( user.LoggedOn.Date );                 (* 1.11 *)
          GEMDOS.GetTime( user.LoggedOn.Time );                 (* 1.11 *)
          UpdateLogFile := FALSE;  (* Logon date already set *)
        ELSIF ch = 'I' THEN
          InitLogFile;
        ELSIF ( ch = 0c ) AND ( scan = 142c ) (* HELP KEY *) THEN
          ClearScreen;
          WriteLine('PROGRAM:        System Log  (SYSLOG.ACC)');
          WriteLine('                Version 1.13, October 24, 1987');
          WriteLn;
          WriteLine('AUTHOR:         John Michael Bennett');
          WriteLine('                12925 - 118th Avenue S.E.');
          WriteLine('                Rainier, Washington 98576');
          WriteLine('                Phone: (206) 446-2427');
          WriteLn;
          WriteLine('LANGUAGE:       TDI Modula-2/ST, Release 3.01a');
          WriteLine('                0272-742796(UK), (214)340-4942(USA)');
          WriteLn;
          WriteLine('SYSTEM:         Atari 1040ST with MIND MINE internal clock, SH204,');
          WriteLine('                SC1224 & SM124.');
          WriteLn;
          WriteLine('PURPOSE:        To keep track of how many hours are spent using the Hard');
          WriteLine('                Disk and to park the Hard Disk when logging off.  Time is');
          WriteLine('                shown in accumulated days, hours, and minutes.');
          WriteLn;
          WriteLn;
          WriteLn;
          WriteLn;
          WriteString('...Press any key...'); Read( ch );
          ClearScreen;
          WriteLine("This program may not be sold for profit.  It is to remain");
          WriteLine("a public domain product.  Copying is permitted without");
          WriteLine("payment of royalty provided that each reproduction is done");
          WriteLine("without alteration and that this notice is included.  There");
          WriteLine("is no warranty of any kind with regard to this software,");
          WriteLine("including, but not limited to its fitness for a paticular");
          WriteLine("purpose.  We shall not be liable for errors contained");
          WriteLine("herein or for incidental or consequential damages in");
          WriteLine("connection with its use.");
          WriteLn;
          WriteLn;
          WriteLn;
          WriteLn;
          WriteString('...Press any key...'); Read( ch );
          ClearScreen;
          Statistics;
          WriteLn;
          WriteLn;
          WriteString('...Press any key...'); Read( ch );
        ELSE
          EXIT;
        END;                                                    (* 1.11 *)
      ELSE                                                      (* 1.11 *)
        EXIT;
      END;                                                      (* 1.11 *)
    END; (* LOOP *)

    IF UpdateLogFile THEN
      GEMDOS.GetDate( CurrentDate );
      GEMDOS.GetTime( CurrentTime );
      Lon  := LONGCARD( user.LoggedOn.Date ) * LONGCARD( 65535 ) +
              LONGCARD( user.LoggedOn.Time );
      Loff := LONGCARD( user.LoggedOff.Date ) * LONGCARD( 65535 ) +
            LONGCARD( user.LoggedOff.Time );

      IF Lon < Loff THEN
        user.LoggedOn.Date := CurrentDate;
        user.LoggedOn.Time := CurrentTime;
      END;
    END;

    len := LONGCARD( SIZE( user ) );
    GEMDOS.Seek( 0, LogHandle, GEMDOS.beginning, fpos );
    GEMDOS.Write( LogHandle, len, ADR( user ) );
    IF GEMDOS.Close( LogHandle ) THEN END;
  ELSE
    WriteString('Unable to open '); WriteString( LogFile ); WriteLn;
    WriteError( LogHandle );
  END;
  RestoreScreen;  (* Restore original screen rez & palette *)   (* 1.13 *)
END LOGON;
    

PROCEDURE LOGOFF;

VAR
  CDays,
  CHours,
  CMinutes,
  CSeconds : CARDINAL;
  TMinutes,
  THours,
  connect  : LONGCARD;
  
BEGIN
  ConfigScreen;
  ShowLogInfo;
  
  GEMDOS.Open( LogFile, 2, LogHandle );
  IF LogHandle < 0 THEN
    WriteError( LogHandle );
    RETURN;
  END;
  
  len := SIZE( user );
  GEMDOS.Read( LogHandle, len, ADR( user ) );
  IF len = SIZE( user ) THEN
    GEMDOS.GetDate( user.LoggedOff.Date );
    GEMDOS.GetTime( user.LoggedOff.Time );

    WITH user DO
      connect := Connected( LoggedOn.Date,  LoggedOn.Time,
                            LoggedOff.Date, LoggedOff.Time );
    END;

    CDays  := CARDINAL( connect DIV 86400 );
    connect := connect - LONGCARD( CDays ) * 86400;
    CHours := CARDINAL( connect DIV 3600 );
    connect := connect - LONGCARD( CHours ) * 3600;
    CMinutes := CARDINAL( connect DIV 60 );
    connect := connect - LONGCARD( CMinutes ) * 60;
    CSeconds := CARDINAL( connect );

    WITH user.ConnectTime DO
      Days := Days + CDays;
      Hours := Hours + CHours;
      Minutes := Minutes + CMinutes;
      Seconds := Seconds + CSeconds;

      IF Seconds > 59 THEN
        Minutes := Minutes + Seconds DIV 60;
        Seconds := Seconds MOD 60;
      END;

      IF Minutes > 59 THEN
        Hours := Hours + Minutes DIV 60;
        Minutes := Minutes MOD 60;
      END;

      IF Hours > 23 THEN
        Days := Days + Hours DIV 24;
        Hours := Hours MOD 24;
      END;
    END; (* WITH *)

    IF NegativeTime THEN
      WriteLine("*** NEGATIVE CONNECT TIME NOT LOGGED ***");
    END;

    WriteString("CONNECT = ");

    IF CDays > 0 THEN
      WriteCard( CDays, 1 ); WriteString(" DAY");
      IF CDays > 1 THEN
        Write("S");
      END;
      WriteString(", ");
    END;
    
    WriteCardUsing( CHours, 2, "0" ); Write(":");
    WriteCardUsing( CMinutes, 2, "0" ); Write(":");
    WriteCardUsing( CSeconds, 2, "0" ); WriteLn; WriteLn;

    IF NegativeTime THEN  (* Set in case connect time was negative *)
      user.LoggedOn.Date := user.LoggedOff.Date;
      user.LoggedOn.Time := user.LoggedOff.Time;
    END;

    user.Sessions := user.Sessions + 1;
    len := LONGCARD( SIZE( user ) );
    GEMDOS.Seek( 0, LogHandle, GEMDOS.beginning, fpos );
    GEMDOS.Write( LogHandle, len, ADR( user ) );
    IF GEMDOS.Close( LogHandle ) THEN END;
  END;  
END LOGOFF;

(* This procedure used to be just statements in the EventLoop procedure *)
PROCEDURE ParkHeads;                                            (* 1.02 *)

CONST
  ParkName    = 'C:\PARK.EXE';

VAR
  ParkPrg     : ARRAY [0 .. 40] OF CHAR;
  ch          : CHAR;
  Result      : INTEGER;
  done        : BOOLEAN;                                        (* 1.02 *)
  str,                                                          (* 1.02 *)
  fstr        : ARRAY [0 .. 80] OF CHAR;                        (* 1.02 *)
  str2        : ARRAY [0 .. 3] OF CHAR;                         (* 1.02 *)
  CurDate,
  CurTime,
  i           : CARDINAL;                                       (* 1.02 *)
  connect,                                                      (* 1.10 *)
  freemem     : LONGCARD;                                       (* 1.02 *)
  a           : ADDRESS;                                        (* 1.02 *)
  A1          : ARRAY [0 .. 127] OF CHAR;
  A2          : ARRAY [0 .. 127] OF CHAR;
  A3          : ARRAY [0 .. 127] OF CHAR;
  Alert       : ARRAY [0 .. 255] OF CHAR;

BEGIN
  Assign( ParkPrg, ParkName );             (* For patch purposes,  1.02 *)
  GEMDOS.Alloc( MAX( LONGCARD ), a );                           (* 1.02 *)
  freemem := LONGCARD( a );                                     (* 1.02 *)
  ConvertToString( freemem, 10, FALSE, str, done );             (* 1.02 *)

  fstr := ' ';                                                  (* 1.02 *)
  FOR i := 1 TO Length( str ) DO                                (* 1.02 *)
    str2[0] := str[ Length( str ) - i ];                        (* 1.02 *)
    str2[1] := 0c;                                              (* 1.02 *)
    Insert( str2, fstr, 0 );                                    (* 1.02 *)
    IF (i MOD 3 = 0) AND (i < Length( str )) THEN               (* 1.02 *)
      Insert( ',', fstr, 0 );                                   (* 1.02 *)
    END;                                                        (* 1.02 *)
  END;                                                          (* 1.02 *)
  A1 := '[3][ ';
  Concat( A1, fstr, A1 );                                       (* 1.02 *)
  Concat( A1, 'bytes free. | ', A1 );

  GEMDOS.Open( LogFile, 2, LogHandle );                         (* 1.10 *)
  IF LogHandle > 0 THEN                                         (* 1.10 *)
    len := SIZE( user );                                        (* 1.10 *)
    GEMDOS.Read( LogHandle, len, ADR( user ) );                 (* 1.10 *)
    IF len = SIZE( user ) THEN                                  (* 1.10 *)
      GEMDOS.GetDate( user.LoggedOff.Date );                    (* 1.10 *)
      GEMDOS.GetTime( user.LoggedOff.Time );                    (* 1.10 *)

      WITH user DO                                              (* 1.10 *)
        connect := Connected( LoggedOn.Date,  LoggedOn.Time,    (* 1.10 *)
                              LoggedOff.Date, LoggedOff.Time ) DIV 60;
      END;                                                      (* 1.10 *)
      fstr := 'Connected ';                                     (* 1.10 *)
      ConvertToString( connect, 10, FALSE, str, done );         (* 1.10 *)
      Concat( fstr, str, fstr );                                (* 1.10 *)
      Concat( fstr, ' minutes. |', fstr );                      (* 1.10 *)
      Concat( A1, fstr, A1 );                                   (* 1.10 *)
    END;                                                        (* 1.10 *)
  END;                                                          (* 1.10 *)

  IF GEMDOS.Close( LogHandle ) THEN END;                        (* 1.10 *)

  A2 := '                      | Park Hard Disk heads |';
  A3 := ' and logoff system?   ][ PARK | CANCEL ]';
  Concat( A1, A2, Alert );
  Concat( Alert, A3, Alert );
  Result := FormAlert( 0, Alert );
  IF Result = 1 THEN
    GrafMouse( MouseOff, NIL );
    LOGOFF;
    GotoXY( 0, 21 );
    WriteString('Parking hard disk heads....');

    REPEAT
      GEMDOS.Exec( GEMDOS.loadExecute, ParkPrg, 0C, 0C, Result );
      IF Result < -1 THEN
        WriteLn;
        WriteString('***** Unable to Load & Execute ');
        WriteString( ParkPrg ); WriteString(' *****'); WriteLn;
        WriteError( Result );

        IF ParkPrg[0] = 'A' THEN                                (* 1.01b *)
          GotoXY( 0, 19 );
          WriteString('Please INSERT SH BOOT-DISK into DRIVE A:');
          WriteLn;
          WriteString('.....Press <RETURN> to continue.....');
          REPEAT
            GEMDOS.RawIn( ch );
          UNTIL ch = 15c;
        ELSE                                                    (* 1.01b *)
          Write('"'); WriteString( ParkPrg ); Write('"');
          WriteString(' not found.  Please find this program ');
          WriteString('and install it as such.'); WriteLn;
          Read( ch );
          RETURN;
        END;
      ELSE
        GotoXY( 0, 21 );
        WriteString("Hard Disk heads are now parked.");
      END;
    UNTIL Result > -2;

    REPEAT
      GotoXY( 0, 19 );
      ReverseOn;  (* Reverse video on *)
      WriteString('TURN OFF THE HARD DISK DRIVE.');
      ReverseOff;
      Pause( 1 ); (* wait 1 second *)

      GotoXY( 0, 19 );
      WriteString('TURN OFF THE HARD DISK DRIVE.');
      Pause( 1 ); (* wait 1 second *)
    UNTIL FALSE;
  END;
END ParkHeads;


PROCEDURE eventloop;

VAR
  pipe        : ARRAY [0 .. 15] OF CARDINAL;
  
BEGIN
  REPEAT
    EventMessage( ADR( pipe ) );
    IF pipe[0] = AccessoryOpen THEN
      IF pipe[4] = CARDINAL( menuid ) THEN
        ParkHeads;
      END;
    END;
  UNTIL FALSE;
END eventloop;
    

(*$PAGE "OUTER BLOCK"*)
BEGIN

(* Increase stack size because we are a big desk accessory *)

  SETREG( 15, ADR( Stack[ StackSize ] ));

  apid := ApplInitialise();
  IF apid >= 0 THEN
    accname := '> Park Hard Disk.. <';
    accname[ 0 ]  := CHR( 3 );
    accname[ 19 ] := CHR( 4 );
    menuid := MenuRegister( apid, accname );

    LOGON;
    ClearScreen;

    eventloop;
    
(*  The following statements will never be executed  *)

    ApplExit;
  END;
END SysLog.
